@@ -1,6 +1,6 @@
-#!/usr/local/bin/python3.9
+#!/usr/local/bin/python3.11
 ###############################################################################
-# $Id: vec_tr_spat.py e4fe7cc06270e5f38dfe78e6785a6bcca4e39e29 2021-04-01 21:02:04 +0300 Idan Miara $
+# $Id$
 #
 # Project:  OGR Python samples
 # Purpose:  Apply a transformation to all OGR geometries.
@@ -36,12 +36,12 @@ from osgeo import ogr
 
 
 def Usage():
-    print('Usage: vec_tr_spat.py [-spat xmin ymin xmax ymax] infile outfile [layer]')
-    print('')
-    return 1
+    print("Usage: vec_tr_spat.py [-spat xmin ymin xmax ymax] infile outfile [layer]")
+    print("")
+    return 2
 
 
-def main(argv):
+def main(argv=sys.argv):
     infile = None
     outfile = None
     layer_name = None
@@ -50,7 +50,7 @@ def main(argv):
     while i < len(argv):
         arg = argv[i]
 
-        if arg == '-spat':
+        if arg == "-spat":
             s_minx = int(argv[i + 1])
             s_miny = int(argv[i + 2])
             s_maxx = int(argv[i + 3])
@@ -67,12 +67,12 @@ def main(argv):
             layer_name = arg
 
         else:
-            Usage()
+            return Usage()
 
         i = i + 1
 
     if outfile is None:
-        Usage()
+        return Usage()
 
     #############################################################################
     # Open the datasource to operate on.
@@ -89,14 +89,14 @@ def main(argv):
     #############################################################################
     # Create output file with similar information.
 
-    shp_driver = ogr.GetDriverByName('ESRI Shapefile')
+    shp_driver = ogr.GetDriverByName("ESRI Shapefile")
     shp_driver.DeleteDataSource(outfile)
 
     shp_ds = shp_driver.CreateDataSource(outfile)
 
-    shp_layer = shp_ds.CreateLayer(in_defn.GetName(),
-                                   geom_type=in_defn.GetGeomType(),
-                                   srs=in_layer.GetSpatialRef())
+    shp_layer = shp_ds.CreateLayer(
+        in_defn.GetName(), geom_type=in_defn.GetGeomType(), srs=in_layer.GetSpatialRef()
+    )
 
     in_field_count = in_defn.GetFieldCount()
 
@@ -153,5 +153,5 @@ def main(argv):
     return 0
 
 
-if __name__ == '__main__':
+if __name__ == "__main__":
     sys.exit(main(sys.argv))
